@@ -1,4 +1,15 @@
+//! **Crate features**
+//!
+//! * `"std"`
+//! Enabled by default. Disable to make the library `#![no_std]`.
+//! * `"nightly"`
+//! Enabled by default. Disable to make the library compatible with stable and beta Rust channels.
+
 #![deny(warnings)]
+#![doc(test(attr(deny(warnings))))]
+#![doc(test(attr(allow(dead_code))))]
+#![doc(test(attr(allow(unused_variables))))]
+
 #![cfg_attr(all(feature="nightly", feature="std"), feature(const_fn_fn_ptr_basics))]
 #![cfg_attr(feature="nightly", feature(const_fn_trait_bound))]
 //#![cfg_attr(feature="nightly", feature(const_trait_impl))]
@@ -8,12 +19,8 @@
 
 #![cfg_attr(not(feature="std"), no_std)]
 
-//! **Crate features**
-//!
-//! * `"std"`
-//! Enabled by default. Disable to make the library `#![no_std]`.
-//! * `"nightly"`
-//! Enabled by default. Disable to make the library compatible with stable and beta Rust channels.
+#[cfg(feature="nightly")]
+include!("doc_test_readme.include");
 
 extern crate alloc;
 #[cfg(feature="std")]
@@ -38,14 +45,16 @@ use core::ops::{Index, IndexMut};
 use core::sync::atomic::{AtomicBool, Ordering};
 use educe::Educe;
 use either::{Either, Left, Right};
+#[cfg(all(feature="std", feature="nightly"))]
+use macro_attr_2018::macro_attr;
+#[cfg(all(feature="std", feature="nightly"))]
+use newtype_derive_2018::NewtypeDeref;
 use phantom_type::PhantomType;
 use rand::rngs::SmallRng;
 use rand::{RngCore, SeedableRng};
 #[cfg(all(feature="std", feature="nightly"))]
 use std::lazy::SyncLazy;
 #[cfg(all(feature="std", feature="nightly"))]
-use std::ops::Deref;
-#[cfg(all(feature="std", feature="nightly"))]
 use std::sync::Mutex;
 
 /// The return type of the [`ComponentClass::lock`](ComponentClass::lock) function.
@@ -212,6 +221,7 @@ pub struct Arena<C: Component> {
 /// ```
 ///
 /// In the `no_std` environment a custom solution should be used to store `ComponentClassToken`.
+#[derive(Debug)]
 pub struct ComponentClassToken<C: ComponentClass> {
     guard_seed_rng: SmallRng,
     _phantom: PhantomType<C>
@@ -454,30 +464,33 @@ impl<C: Component> IndexMut<Id<C>> for Arena<C> {
     }
 }
 
-/// Helps to store [`ComponentClassToken`](ComponentClassToken) in a static.
-///
-/// # Examples
-///
-/// ```rust
-/// # use macro_attr_2018::macro_attr;
-/// # use components_arena::{Component, ComponentClassMutex, Arena};
-/// #
-/// macro_attr! {
-///     #[derive(Component!)]
-///     struct MyComponent { /* ... */ }
-/// }
-///
-/// static MY_COMPONENT: ComponentClassMutex<MyComponent> = ComponentClassMutex::new();
-///
-/// // ...
-///
-/// # fn main() {
-/// let mut arena = Arena::new(&mut MY_COMPONENT.lock().unwrap());
-/// # let id = arena.insert(|id| (MyComponent { /* ... */ }, id));
-/// # }
-/// ```
 #[cfg(all(feature="std", feature="nightly"))]
-pub struct ComponentClassMutex<C: ComponentClass>(SyncLazy<Mutex<ComponentClassToken<C>>>);
+macro_attr! {
+    /// Helps to store [`ComponentClassToken`](ComponentClassToken) in a static.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # use macro_attr_2018::macro_attr;
+              /// # use components_arena::{Component, ComponentClassMutex, Arena};
+    /// #
+    /// macro_attr! {
+    ///     #[derive(Component!)]
+    ///     struct MyComponent { /* ... */ }
+    /// }
+    ///
+    /// static MY_COMPONENT: ComponentClassMutex<MyComponent> = ComponentClassMutex::new();
+    ///
+    /// // ...
+    ///
+    /// # fn main() {
+    /// let mut arena = Arena::new(&mut MY_COMPONENT.lock().unwrap());
+    /// # let id = arena.insert(|id| (MyComponent { /* ... */ }, id));
+    /// # }
+    /// ```
+    #[derive(Debug, NewtypeDeref!)]
+    pub struct ComponentClassMutex<C: ComponentClass>(SyncLazy<Mutex<ComponentClassToken<C>>>);
+}
 
 #[cfg(all(feature="std", feature="nightly"))]
 impl<C: ComponentClass> ComponentClassMutex<C> {
@@ -491,13 +504,6 @@ impl<C: ComponentClass> ComponentClassMutex<C> {
     }
 }
 
-#[cfg(all(feature="std", feature="nightly"))]
-impl<C: ComponentClass> Deref for ComponentClassMutex<C> {
-    type Target = Mutex<ComponentClassToken<C>>;
-
-    fn deref(&self) -> &Self::Target { self.0.deref() }
-}
-
 /// [Macro attribute](https://crates.io/crates/macro-attr-2018) for deriving [`Component`](trait@Component) trait.
 ///
 /// # Examples
@@ -556,7 +562,7 @@ macro_rules! Component {
         $($body:tt)*
     ) => {
         $crate::generics_parse! {
-            $crate::Component {
+            $crate::Component_impl {
                 @impl [$vis] [$name] [$($class)?]
             }
             $($body)*
@@ -568,21 +574,27 @@ macro_rules! Component {
         $($body:tt)*
     ) => {
         $crate::generics_parse! {
-            $crate::Component {
+            $crate::Component_impl {
                 @impl [$vis] [$name] [$($class)?]
             }
             $($body)*
         }
     };
+}
+
+
+#[doc(hidden)]
+#[macro_export]
+macro_rules! Component_impl {
     (
         @impl [$vis:vis] [$name:ident] [] [] [] [] $($body:tt)*
     ) => {
-        $crate::Component! { @self [$name] }
+        $crate::Component_impl! { @self [$name] }
     };
     (
         @impl [$vis:vis] [$name:ident] [$class:ident] [] [] [] $($body:tt)*
     ) => {
-        $crate::Component! { @class [$vis] [$name] [$class] [] [] [] }
+        $crate::Component_impl! { @class [$vis] [$name] [$class] [] [] [] }
     };
     (
         @impl [$vis:vis] [$name:ident] [] [$($g:tt)+] [$($r:tt)+] [$($w:tt)*] $($body:tt)*
@@ -595,7 +607,7 @@ macro_rules! Component {
     (
         @impl [$vis:vis] [$name:ident] [$class:ident] [$($g:tt)+] [$($r:tt)+] [$($w:tt)*] $($body:tt)*
     ) => {
-        $crate::Component! { @class [$vis] [$name] [$class] [$($g)+] [$($r)+] [$($w)*] }
+        $crate::Component_impl! { @class [$vis] [$name] [$class] [$($g)+] [$($r)+] [$($w)*] }
     };
     (
         @self [$name:ident]
@@ -626,14 +638,21 @@ macro_rules! Component {
     };
 }
 
-/// [Macro attribute](https://crates.io/crates/macro-attr-2018) for deriving [`ComponentId`](trait@ComponentId) trait.
+/// Obsolete. Use [`NewtypeComponentId`] instead.
+#[macro_export]
+macro_rules! ComponentId {
+    ($($token:tt)*) => { $crate::NewtypeComponentId! { $($token)* } };
+}
+
+/// [Macro attribute](https://crates.io/crates/macro-attr-2018)
+/// for deriving [`ComponentId`](trait@ComponentId) trait.
 ///
 /// # Examples
 ///
 /// ```rust
 /// # use educe::Educe;
 /// # use macro_attr_2018::macro_attr;
-/// use components_arena::{Component, Id, ComponentId};
+/// use components_arena::{Component, Id, NewtypeComponentId};
 /// use phantom_type::PhantomType;
 ///
 /// # macro_attr! {
@@ -645,63 +664,39 @@ macro_rules! Component {
 /// # }
 /// #
 /// macro_attr! {
-///     #[derive(ComponentId!)]
-///     #[derive(Educe)]
+///     #[derive(Educe, NewtypeComponentId!)]
 ///     #[educe(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd)]
 ///     pub struct Item<Tag, X>(Id<ItemNode<Tag>>, PhantomType<X>);
 /// }
 /// ```
 #[macro_export]
-macro_rules! ComponentId {
+macro_rules! NewtypeComponentId {
     (
         ()
         $vis:vis struct $name:ident $($body:tt)*
     ) => {
         $crate::generics_parse! {
-            $crate::ComponentId {
-                @struct [$vis] [$name]
+            $crate::NewtypeComponentId_impl {
+                [$name]
             }
             $($body)*
         }
     };
+}
+
+#[doc(hidden)]
+#[macro_export]
+macro_rules! NewtypeComponentId_impl {
     (
-        @struct [$vis:vis] [$name:ident] [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
-        ($id:ty $(, $($phantom:ty),+ $(,)?)?);
-    ) => {
-        $crate::ComponentId! {
-            @impl [$vis] [$name] [$($g)*] [$($r)*] [$($w)*]
-            []
-            [$($($phantom),+)?]
-        }
-    };
-    (
-        @struct [$vis:vis] [$name:ident] [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
-        $($body:tt)*
-    ) => {
-        $crate::std_compile_error!("'ComponentId' deriving is supported for non-empty tuple structs only");
-    };
-    (
-        @impl [$vis:vis] [$name:ident] [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
-        [$($phantom_args:tt)*]
-        [$phantom:ty $(, $($other_phantoms:tt)+)?]
-    ) => {
-        $crate::ComponentId! {
-            @impl [$vis] [$name] [$($g)*] [$($r)*] [$($w)*]
-            [
-                $($phantom_args)*
-                $crate::std_default_Default::default(),
-            ]
-            [$($($other_phantoms)+)?]
-        }
-    };
-    (
-        @impl [$vis:vis] [$name:ident] [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
-        [$($phantom_args:tt)*]
-        []
+        [$name:ident] [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
+        ($(pub)? $id:ty $(, $(pub)? $phantom:ty)* $(,)?);
     ) => {
         impl $($g)* $crate::ComponentId for $name $($r)* $($w)* {
             fn from_raw(raw: $crate::RawId) -> Self {
-                $name($crate::Id::from_raw(raw), $($phantom_args)*)
+                $name(
+                    $crate::Id::from_raw(raw)
+                    $(, <$phantom as $crate::std_default_Default>::default())*
+                )
             }
 
             fn into_raw(self) -> $crate::RawId {
@@ -709,6 +704,12 @@ macro_rules! ComponentId {
             }
         }
     };
+    (
+        [$name:ident] [$($g:tt)*] [$($r:tt)*] [$($w:tt)*]
+        $($body:tt)*
+    ) => {
+        $crate::std_compile_error!("'NewtypeComponentId' deriving is supported for non-empty tuple structs only");
+    };
 }
 
 #[cfg(test)]
@@ -787,21 +788,21 @@ mod test {
     }
 
     macro_attr! {
-        #[derive(ComponentId!)]
+        #[derive(NewtypeComponentId!)]
         #[derive(Educe)]
         #[educe(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd)]
         struct IdWrap1(Id<Test>);
     }
 
     macro_attr! {
-        #[derive(ComponentId!)]
+        #[derive(NewtypeComponentId!)]
         #[derive(Educe)]
         #[educe(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd)]
         struct IdWrap2<X>(Id<Test>, PhantomType<X>);
     }
 
     macro_attr! {
-        #[derive(ComponentId!)]
+        #[derive(NewtypeComponentId!)]
         #[derive(Educe)]
         #[educe(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd)]
         struct IdWrap3<X, Y: Copy>(Id<Test>, PhantomType<X>, PhantomType<Y>);
